import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.ArrayList;

import javax.comm.CommPortIdentifier;
import javax.comm.SerialPort;
import javax.swing.JProgressBar;

public class Bootloader {
	
	public final static char KEY_ENTER = 13;
	public final static char COMMAND_CONTINUE ='c';
	public final static char COMMAND_FAIL = 'f';
	public final static char COMMAND_DONE = 'a';
	public final static int TIMEOUTSTART =	2;
	public final static int TIMEOUT	=	500 ;// 0.5s
	public final static int TIMEOUTP =	10000;// 10s
	
	private boolean isInterrupted = false;
	
	public int errAddr = 0;
	
	int crc;
	char Passwd[] = "\370\360\340\300\374".toCharArray();	// F8,F0,E0,C0,FC
	
	int baudrate;
	String comPort;
	CommPortIdentifier cpi;
	SerialPort serss;
	InputStream in;
	OutputStream out;
	
	public Bootloader(String comPort,int baudrate){
		this.baudrate = baudrate;
		this.comPort = comPort;
	}
	
	public int read_crc() throws IOException{
		long[] val = new long[1];
		int c;

		  sendstring( "CRC".toCharArray() );

		  c = crc;
		  crc = 0;

		  for(;;){
		    switch( read_value( TIMEOUT, val ) ){
		      case 1: if( val[0] == c )			// CRC read
				return 0;
			      return -2;
		      case -1: return -1;                      	// Timeout
		    }
		  }
	}
	
	private void get_crc(char d )
	{
	  int i;

	  crc ^= d;
	  for( i = 8; i > 0; i-- ){
	    crc = (crc >> 1) ^ ((crc & 1)>0 ? 0xA001 : 0 );
	  }
	}

	public int program(File file,boolean vertify,boolean eeprom,JProgressBar progress) throws IOException{
		char[] data = new char[65536];
		int returncode;
		long[] val = new long[1];
		int[] firstaddr=new int[1],lastaddr=new int[1];
		int addr = 0;
		char d;
		
		returncode = readhex(file,data,firstaddr,lastaddr);
		if (returncode != 0) {
			if (returncode == -1) {
				System.out.println("Unerwartetes Dateiende - Datei defekt\n");
			}else if(returncode == -2){
				System.out.println("Keine Hex-Datei oder Datei defekt\n");
			}
			return returncode;
		}
		
		
		if (eeprom) {
			sendstring("DEM8".toCharArray());
		}else{
			sendstring("DM8".toCharArray());
		}
		if (read_value(TIMEOUT,val) != 0) {
			System.out.println("Keine Antwort");
			return -3;		//	Keine Antwort
		}
		if (vertify) {
			System.out.print("Vertifying ");
			sendstring(("VER"+firstaddr[0]).toCharArray());
		}else{
			System.out.print("Programming ");
			sendstring(("PRO"+firstaddr[0]).toCharArray());
		}
		if (eeprom) {
			System.out.println("EEProm");
		}else{
			System.out.println("Flash");
		}
		
		if (progress != null) {
			progress.setMinimum(firstaddr[0]);
			progress.setMaximum(lastaddr[0]-1);
		}
		for (addr = firstaddr[0];; addr++) {
			if (progress != null) {
				progress.setValue(addr);
			}
			d = data[addr];
			senden(d);
						
			if (d == 165) {		// wenn d = 0xA5, 0x00 senden um byte zu besttigen
				d = 0;
				senden(d);
			}
			try{
				if ((addr)%(((lastaddr[0]-firstaddr[0])/33)) == 0) {
					System.out.print('#');
				}
			}catch(Exception e){
				
			}
			
			if ((addr & 511)  == 511 ) {
				returncode = read_value(TIMEOUT,val);
				if (returncode != 2) {
					System.out.println("\nError at address: "+addr);
					errAddr = addr;
					return -4;
				}
			}
			if (addr == lastaddr[0]) {
				senden((char)165);
				senden((char)255);
				if (read_value(TIMEOUT,val) != 0) {
					errAddr = addr;
					return -4;		//	Keine Antwort
				}
				System.out.println("\nDone !");
				return 0;
			}
		}		
	}
	
	public void setBaudrate(int baudrate){
		this.baudrate = baudrate;
	}
	
	public void setComport(String comPort){
		this.comPort = comPort;
	}
	
	public synchronized void interrupt(){
		isInterrupted = true;
		try {
			this.wait(20);
		} catch (InterruptedException e) {
		}
		try {
			disconnect();
		} catch (IOException e) {
		}
		
	}
	
	public int read(){
		
		return 0;
	}
	
	public boolean connect() throws Exception{
		isInterrupted = false;
		cpi = CommPortIdentifier.getPortIdentifier(comPort);
		serss = (SerialPort) cpi.open("Bootloader Flash Utility",2000);
		
		in = serss.getInputStream();
		out = serss.getOutputStream();
		
		serss.setSerialPortParams(baudrate,
                SerialPort.DATABITS_8,
                SerialPort.STOPBITS_1 ,
                SerialPort.PARITY_NONE);
		
		serss.notifyOnDataAvailable(true);
		
		
		System.out.print("Port: "+ comPort +" at "+baudrate+" Baud: ");
		
		
		while(true){
			
		    if( (System.in.available()!=0) || isInterrupted){
		      System.out.print("Aborted\n\n");
		      return false;
		    }
		    sendstring( Passwd );
		    if( empfang( 100 ) == COMMAND_DONE ){
		      System.out.print("Connected\n");		
		      crc = 0;
		      in.skip(in.available());	// Clear buffer
		      return true;
		    }
		  }
		
		
	}
	
	public void disconnect() throws IOException{
		sendstring("RES".toCharArray());
		serss.close();
	}
	
	public void helptext(){
		System.out.print( "/?\t\t Get this help message\n"+
				  "/Bnnnn\t\t Define baud rate\n"+
				  "/Cn\t\t Define serial port n = 1..4\n"+
				  "/PFname,Ename\t Perform Program\n"+
				  "/VFname,Ename\t Perform Verify\n"+
				  //"/RFname,Ename\t Perform Read\n"
				  "/D\t\t Debug Mode\n"+
				  "Press Enter ! \n" );
	}
	
	public String read_sig() throws IOException{
		long[] signature = new long[1];
		String dev = "";

		sendstring( "RSI".toCharArray() );
		while( (read_value( TIMEOUT, signature ) != 1) && !isInterrupted );
		if( signature[0] != 0 ){
			  
			if (signature[0] == 2003719) {
				dev = "ATMega8";
			}else if(signature[0] == 2003971){
				dev = "ATMega16";
			}else if (signature[0] == 2003972) {
				dev = "ATMega162";
			}else if (signature[0] == 2004225) {
				dev = "ATMega323";
			}
			System.out.print( "Signature: "+signature[0]+" Device: "+ dev+"\n" );
		 
		}
		return signature[0]+" Device: "+dev;
	}
	
	private int read_value( int timeout, long[] val ) throws IOException{
	  long newval = 0;
	  int i, v = 0;

	  for(;;){
		  i = empfang( timeout );
		  switch( i ){
		  	case '0':
		  	case '1':
		  	case '2':
		  	case '3':
		  	case '4':
		  	case '5':
		  	case '6':
		  	case '7':
		  	case '8':
		  	case '9': i += 'A' - '0' - 10;
		  	case 'A':
		  	case 'B':
		  	case 'C':
		  	case 'D':
		  	case 'E':
		  	case 'F': newval = newval * 16 + i - 'A' + 10;
			v = 1;
			break;

		  	case COMMAND_CONTINUE:
		  		return 2;		// 2

		  	case COMMAND_DONE:
		  		val[0] = newval;
		  		return v;            	// 0, 1

		  	case COMMAND_FAIL:
		  		val[0] = newval;
		  		return -2 - v;		// -2, -3

		  	default:	return -1;
		  }

	  	}
	}
	
	private int empfang (int timeout) throws IOException{
		int i;
		long start = System.currentTimeMillis();
		
		do {
			if (in.available() > 0) {
				i = in.read();
				return i;
			}
		} while ((System.currentTimeMillis() - start) < timeout);
		return -1;
	}
	
	private void sendstring(char[] text) throws IOException{
		  for (int i = 0; i < text.length; i++) {
			  senden( text[i] );
		  }
		  senden( KEY_ENTER );			// command end
	}
	
	private void senden(char c) throws IOException
	{
	  out.write(c);
	  get_crc( c );					// calculate transmit CRC
	}
	
	public int readhex(File file, char[] data,int[] firstaddr,int[] lastaddr) throws IOException{
		FileInputStream fis = new FileInputStream(file);
		char[] hexline = new char[524];
		char[] dataline = new char[512];
		int datalinenr = 0;
		int datalength;
		int addr;
		int rtype;
		int crc;
		int dp = 0;
		long bytesread = 0;
		String number = "";
		
		int temp;
		int x = 0;
		
		
		while(true){
			hexline = new char[524];
			x=0;
			while (true) {
				temp = fis.read();
				bytesread++;
				if ( ((char)temp) == 10) {		// erkennung von LF als Zeilenende
					break;
				}else if(temp == -1){
					return -1; 			// EOF
				}else{
					hexline[x] = (char)temp;
					x++;
				}
			}
			
			if (hexline[0] != ':') {
				return -2;		// No Hex record
			}
			
			int hlm = 1;
			int i;
			number = "";
			for (i = hlm; i < hlm+2; i++) {
				number+= hexline[i];
			}
			hlm = i;
			datalength = hex2int(number.toCharArray());
			
			
			number = "";
			for (i = hlm; i < hlm+4; i++) {
				number += hexline[i];
			}
			hlm = i;
			addr = hex2int(number.toCharArray());
			
			
			number = "";
			for (i = hlm; i < hlm+2; i++) {
				number += hexline[i];
			}
			hlm = i;
			rtype = hex2int(number.toCharArray());
			
			
			number = "";
			int y = 0;
			for (i = hlm; i < hlm+(datalength*2); i++) {
				dataline[y] = hexline[i];
				y++;
			}
			hlm = i;
			
			
			number = "";
			for (i = hlm; i < hlm+2; i++) {
				number += hexline[i];
			}
			hlm = i;
			crc = hex2int(number.toCharArray());
			
			
			dp = 0;
			if (rtype == 0) {
				for (int j = 0; j < datalength*2; j+=2) {
					if (datalinenr == 0) {
						firstaddr[0] = addr;
					}
					datalinenr++;
					number = "";
					number+= dataline[j];
					number+= dataline[j+1];
					data[addr+dp] = (char)hex2int(number.toCharArray());
					dp++;
					lastaddr[0] = addr+datalength;
				}
			}else if(rtype == 1){
				return 0; 		// Last Record of File
			}
			
			//System.out.println("Datenlnge: "+datalength+" Adresse: "+addr+" Recordtype: "+rtype+" CRC: "+crc);
		}
	}
	
	private int hex2int(char[] c){
		int weight;
		int value = 0;
		
		for (int i = c.length-1,x = 0; i >= 0 ; i--,x++) {
			weight = (int)Math.pow(16,x);
			int temp;
			if ( (c[i] >= '0') && (c[i] <= '9') ) {
				temp = (c[i]-'0')*weight;
			}else if( (c[i] >= 'A') && (c[i] <= 'F')){
				temp = (c[i]-'A'+10)*weight;
			}else{
				return -1;		// No Hex Number
			}
			value+=temp;
		}
		return value;
	}
	
}
