import java.io.File;
import java.io.IOException;
import java.util.StringTokenizer;

import javax.comm.CommPort;
import javax.comm.CommPortIdentifier;

public class Bootloader_main {
	public static void main(String[] args){
		boolean showGUI = false;
		boolean printHelp = false;
		String FlashFileName = "";
		String EepromFileName = "";
		boolean p = false;
		boolean v = false;
		boolean r = false;
		String comPort = "COM1";		
		int baudrate = 115200;
		Bootloader bootloader = null;
		
		for (int i = 0; i < args.length; i++) {
			if (args[i].charAt(0) == '-') {
				args[i] = args[i].substring(1,args[i].length());	// Remove first Char ('-')
				char keyChar = Character.toLowerCase(args[i].charAt(0));
				String arg = args[i].substring(1,args[i].length());  // Remove fist Char (keyChar)
				
				if (keyChar == 'g') {
					showGUI = true;
				}else if(keyChar == 'b'){
					baudrate = Integer.parseInt(arg); //Parse to Int
				}else if(keyChar == 'c'){
					comPort = arg;
				}else if(keyChar == 'p'){
					p = true;
					StringTokenizer stringTok = new StringTokenizer(arg,",");
					if (stringTok.hasMoreTokens()) {
						FlashFileName = stringTok.nextToken();
					}
					if (stringTok.hasMoreTokens()) {
						EepromFileName = stringTok.nextToken();
					}
				}else if(keyChar == 'v'){
					v = true;
					StringTokenizer stringTok = new StringTokenizer(arg,",");
					if (stringTok.hasMoreTokens()) {
						FlashFileName = stringTok.nextToken();
					}
					if (stringTok.hasMoreTokens()) {
						EepromFileName = stringTok.nextToken();
					}
				}else if(keyChar == 'r') {
					r = true;
					StringTokenizer stringTok = new StringTokenizer(arg,",");
					if (stringTok.hasMoreTokens()) {
						FlashFileName = stringTok.nextToken();
					}
					if (stringTok.hasMoreTokens()) {
						EepromFileName = stringTok.nextToken();
					}
				}else if(keyChar == '?'){
					printHelp = true;
				}
			}
		}
		
		bootloader = new Bootloader(comPort,baudrate);
		
		
		
		if (showGUI) {
			Bootloader_GUI gui = new Bootloader_GUI(bootloader);
			gui.setVisible(true);
		}else{
			if (printHelp) {
				bootloader.helptext();
			}
			try {
				if (bootloader.connect()) {
					bootloader.read_sig();
					File flashFile = new File(FlashFileName);
					if (flashFile.exists()) {
						bootloader.program(flashFile,v,false,null);
						if (bootloader.read_crc() == 0) {
							System.out.println("Done!\nCRC Pass\n");
						}else{
							System.out.println("CRC error\nFailed!\n");
						}
					}
					File eepromFile = new File(EepromFileName);
					if (eepromFile.exists()) {
						bootloader.program(flashFile,false,true,null);
						if (bootloader.read_crc() == 0) {
							System.out.println("Done!\nCRC Pass\n");
						}else{
							System.out.println("CRC error\nFailed!\n");
						}
					}
					System.out.println();
				}				
				bootloader.disconnect();
			} catch (Exception e) {
				//System.out.println(e.toString());
				e.printStackTrace();
			}
		}
		
	}
}
